﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2021. All rights reserved.
//
//*********************************************************

using System;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Windows.Devices.SerialCommunication;
using Windows.Storage.Streams;

namespace SampleDisplayControl.Communications
{
    internal class SerialCommunication
    {
        private SerialDevice currentDevice;
        private DataWriter dataWriter;
        private DataReader dataReader;

        private static readonly SemaphoreSlim semaphore = new SemaphoreSlim(1, 1);

        public async Task<bool> OpenAsync(string id)
        {
            currentDevice = await SerialDevice.FromIdAsync(id);

            if (currentDevice == null)
            {
                return false;
            }

            currentDevice.ReadTimeout = TimeSpan.FromMilliseconds(500);
            currentDevice.WriteTimeout = TimeSpan.FromMilliseconds(500);
            dataWriter = new DataWriter(currentDevice.OutputStream);
            dataReader = new DataReader(currentDevice.InputStream);
            return true;
        }

        public void Close()
        {
            if (currentDevice == null)
            {
                return;
            }

            currentDevice.Dispose();
            currentDevice = null;

            dataReader?.Dispose();
            dataReader = null;

            dataWriter?.Dispose();
            dataWriter = null;
        }

        protected async Task<string> WriteAndReadAsync(string cmd)
        {
            await semaphore.WaitAsync().ConfigureAwait(false);
            try
            {
                if (WriteData(cmd))
                {
                    return await ReadDataAsync(cmd).ConfigureAwait(false);
                }
            }
            finally
            {
                semaphore.Release();
            }

            return string.Empty;
        }

        private bool WriteData(string cmd)
        {
            if (dataWriter == null)
            {
                return false;
            }

            using (var cts = new CancellationTokenSource(TimeSpan.FromMilliseconds(1500)))
            {
                try
                {
                    dataWriter.WriteString(cmd + Environment.NewLine);
                    dataWriter.StoreAsync().AsTask(cts.Token);

                    return true;
                }
                catch (OperationCanceledException)
                {
                    Debug.WriteLine("Write Timeout");
                }
                catch (Exception e)
                {
                    Debug.WriteLine("Write Exception : " + e.Message);
                }
            }
            return false;
        }

        private async Task<string> ReadDataAsync(string cmd)
        {
            if (dataReader == null)
            {
                return string.Empty;
            }

            using (var cts = new CancellationTokenSource(TimeSpan.FromMilliseconds(1500)))
            {
                try
                {
                    Task<uint> loadAsyncTask = dataReader.LoadAsync(1024).AsTask(cts.Token);
                    uint bytesRead = await loadAsyncTask.ConfigureAwait(false);

                    if (bytesRead > 0)
                    {
                        var readData = dataReader.ReadString(bytesRead);
                        return GetReadData(readData, cmd);
                    }
                }
                catch (OperationCanceledException)
                {
                    Debug.WriteLine("Read Timeout");
                }
                catch (Exception e)
                {
                    Debug.WriteLine("Read Exception : " + e.Message);
                }
            }
            return string.Empty;
        }

        private string GetReadData(string data, string cmd)
        {
            // The return header of the command is "\r\n".
            string returnHeader = cmd + "\r\n";
            // THe return termination code is "\r\n\r\n".
            string returnTerminationCode = "\r\n\r\n";

            data = data.Remove(0, data.IndexOf(returnHeader) + returnHeader.Length);
            data = data.Remove(data.IndexOf(returnTerminationCode));
            return data.Trim();
        }
    }
}
