﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2021. All rights reserved.
//
//*********************************************************

using SampleDisplayControl.Communications;
using SampleDisplayControl.Watchers;
using System;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.UI.Core;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace SampleDisplayControl
{
    public sealed partial class MainPage : Page
    {
        private readonly MoverioWatcher moverioWatcher = new MoverioWatcher();
        private readonly DisplayCommunication displayCommunication = new DisplayCommunication();
        private string currentDeviceInformationId = string.Empty;

        public MainPage()
        {
            InitializeComponent();
            ApplicationView.GetForCurrentView().SetPreferredMinSize(new Size(400, 200));
            ApplicationView.PreferredLaunchViewSize = new Size(400, 200);
            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            Application.Current.Resuming += Current_Resuming;
            Application.Current.Suspending += Current_Suspending;

            StartCommunication();
        }

        protected override void OnNavigatingFrom(NavigatingCancelEventArgs e)
        {
            Application.Current.Resuming -= Current_Resuming;
            Application.Current.Suspending -= Current_Suspending;

            ClosedCommunication();
        }

        private void Current_Resuming(object sender, object e)
        {
            StartCommunication();
        }

        private void Current_Suspending(object sender, SuspendingEventArgs e)
        {
            ClosedCommunication();
        }

        private void StartCommunication()
        {
            moverioWatcher.Add += MoverioWatcher_AddAsync;
            moverioWatcher.Remove += MoverioWatcher_RemoveAsync;
            moverioWatcher.Start();
        }

        private void ClosedCommunication()
        {
            moverioWatcher.Stop();
            moverioWatcher.Add -= MoverioWatcher_AddAsync;
            moverioWatcher.Remove -= MoverioWatcher_RemoveAsync;
            displayCommunication.Close();
            currentDeviceInformationId = string.Empty;
            Enable2D3DButtons(false);
            EnableMuteButtons(false);
            EnableAutoSleepButtons(false);
            EnableManualSleepButtons(false);
            EnablePixcellShiftControls(false);
        }

        private async void MoverioWatcher_AddAsync(object sender, MoverioInfo e)
        {
            if (string.IsNullOrEmpty(currentDeviceInformationId))
            {
                currentDeviceInformationId = e.id;
                if (await displayCommunication.OpenAsync(currentDeviceInformationId).ConfigureAwait(false))
                {
                    await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                    {
                        SetMoverioTypeDisplay(e.type);
                        Enable2D3DButtons(true);
                        EnableMuteButtons(true);
                        EnableAutoSleepButtons(true);
                        EnableManualSleepButtons(true);
                        EnablePixcellShiftControls(true);
                    });
                }
            }
        }

        private async void MoverioWatcher_RemoveAsync(object sender, MoverioInfo e)
        {
            if (currentDeviceInformationId.Equals(e.id))
            {
                displayCommunication.Close();
                currentDeviceInformationId = string.Empty;
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    Enable2D3DButtons(false);
                    EnableMuteButtons(false);
                    EnableAutoSleepButtons(false);
                    EnableManualSleepButtons(false);
                    EnablePixcellShiftControls(false);
                });
            }
        }

        private void Get2D3D_Click(object sender, RoutedEventArgs _)
        {
            var mode = displayCommunication.GetModeAsync().Result;
            if (string.IsNullOrEmpty(mode))
            {
                Current2D3D.Text = "Error";
            }
            else
            {
                Current2D3D.Text = mode;
            }
        }

        private async void Set2D3D_Click(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;
            var mode = button.Content as string;
            if (await displayCommunication.SetModeAsync(mode).ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    Enable2D3DButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private void GetMute_Click(object sender, RoutedEventArgs _)
        {
            var mode = displayCommunication.GetMuteAsync().Result;
            if (string.IsNullOrEmpty(mode))
            {
                CurrentMute.Text = "Error";
            }
            else
            {
                CurrentMute.Text = mode;
            }
        }

        private async void SetMute_Click(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;
            var mode = button.Content as string;
            if (await displayCommunication.SetMuteAsync(mode).ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    EnableMuteButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private void GetAutoSleep_Click(object sender, RoutedEventArgs _)
        {
            var mode = displayCommunication.GetAutoSleepAsync().Result;
            if (string.IsNullOrEmpty(mode))
            {
                CurrentAutoSleep.Text = "Error";
            }
            else
            {
                CurrentAutoSleep.Text = mode;
            }
        }

        private async void SetAutoSleep_Click(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;
            var mode = button.Content as string;
            if (await displayCommunication.SetAutoSleepAsync(mode).ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    EnableAutoSleepButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private void GetManualSleep_Click(object sender, RoutedEventArgs _)
        {
            var mode = displayCommunication.GetManualSleepAsync().Result;
            if (string.IsNullOrEmpty(mode))
            {
                CurrentManualSleep.Text = "Error";
            }
            else
            {
                CurrentManualSleep.Text = mode;
            }
        }

        private async void SetManualSleep_Click(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;
            var mode = button.Content as string;
            if (await displayCommunication.SetManualSleepAsync(mode).ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    EnableManualSleepButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private void GetPixcellShift_Click(object sender, RoutedEventArgs _)
        {
            var value = displayCommunication.GetDisplayPixcellShiftAsync().Result;
            if (string.IsNullOrEmpty(value))
            {
                CurrentPixcellShift.Text = "Error";
            }
            else
            {
                CurrentPixcellShift.Text = value;
            }
        }

        private async void SetPixcellShift_Click(object sender, RoutedEventArgs _)
        {
            var selectValue = ComboBoxPixcellShift.SelectedValue.ToString();
            if (int.TryParse(selectValue, out int setValue))
            {
                await displayCommunication.SetDisplayPixcellShiftAsync(setValue).ConfigureAwait(false);
            }
        }

        private void SetMoverioTypeDisplay(MoverioType type)
        {
            var modeBT40 = (MoverioType.BT40 == type || MoverioType.BT45C == type) ? Visibility.Visible : Visibility.Collapsed;

            StackPanelGetMute.Visibility = modeBT40;
            StackPanelSetMute.Visibility = modeBT40;
            StackPanelGetAutoSleep.Visibility = modeBT40;
            StackPanelSetAutoSleep.Visibility = modeBT40;
            StackPanelGetManualSleep.Visibility = modeBT40;
            StackPanelSetManualSleep.Visibility = modeBT40;
            StackPanelGetPixcellShift.Visibility = modeBT40;
            StackPanelSetPixcellShift.Visibility = modeBT40;


            if (MoverioType.BT40 == type || MoverioType.BT45C == type)
            {
                ApplicationView.GetForCurrentView().SetPreferredMinSize(new Size(400, 670));
                ApplicationView.PreferredLaunchViewSize = new Size(400, 670);
                ApplicationView.GetForCurrentView().TryResizeView(new Size { Width = 400, Height = 670 });
            }
            else
            {
                ApplicationView.GetForCurrentView().SetPreferredMinSize(new Size(400, 200));
                ApplicationView.PreferredLaunchViewSize = new Size(400, 200);
                ApplicationView.GetForCurrentView().TryResizeView(new Size { Width = 400, Height = 200 });
            }
            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;
        }

        private void Enable2D3DButtons(bool enable)
        {
            ButtonGet.IsEnabled = enable;
            Button2D.IsEnabled = enable;
            Button3D.IsEnabled = enable;
        }

        private void EnableMuteButtons(bool enable)
        {
            ButtonMuteGet.IsEnabled = enable;
            ButtonMuteOn.IsEnabled = enable;
            ButtonMuteOff.IsEnabled = enable;
        }

        private void EnableAutoSleepButtons(bool enable)
        {
            ButtonAutoSleepGet.IsEnabled = enable;
            ButtonAutoSleepOn.IsEnabled = enable;
            ButtonAutoSleepOff.IsEnabled = enable;
        }

        private void EnableManualSleepButtons(bool enable)
        {
            ButtonManualSleepGet.IsEnabled = enable;
            ButtonManualSleepOn.IsEnabled = enable;
            ButtonManualSleepOff.IsEnabled = enable;
        }

        private void EnablePixcellShiftControls(bool enable)
        {
            ButtonPixcellShiftGet.IsEnabled = enable;
            ComboBoxPixcellShift.IsEnabled = enable;
            ButtonPixcellShiftSet.IsEnabled = enable;
        }
    }
}
