﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2019. All rights reserved.
//
//*********************************************************

using System;
using System.Diagnostics;
using System.IO.Ports;

namespace SampleBrightnessControl.Communications
{
    internal abstract class SerialCommunication
    {
        private readonly int Timeout = 500; // Milliseconds

        private readonly SerialPort serialPort = new SerialPort();
        private readonly object serialPort_lock = new object();

        public bool Open(string portName)
        {
            if (string.IsNullOrEmpty(portName))
            {
                return false;
            }

            if (serialPort.IsOpen)
            {
                Close();
            }

            try
            {
                serialPort.PortName = portName;
                serialPort.BaudRate = 9600;
                serialPort.Parity = Parity.None;
                serialPort.DataBits = 8;
                serialPort.StopBits = StopBits.One;
                serialPort.Handshake = Handshake.None;

                // A value that represents the end of a line of Moverio is "\r\n".
                serialPort.NewLine = "\r\n";

                serialPort.ReadTimeout = Timeout;
                serialPort.WriteTimeout = Timeout;

                serialPort.Open();
            }
            catch (Exception e)
            {
                Debug.WriteLine("Error : " + e.Message);
                return false;
            }

            return true;
        }

        public void Close()
        {
            serialPort.Close();
        }

        protected string WriteAndRead(string cmd)
        {
            lock (serialPort_lock)
            {
                if (WriteData(cmd))
                {
                    return ReadData(cmd);
                }
            }

            return string.Empty;
        }

        private bool WriteData(string cmd)
        {
            if (!serialPort.IsOpen)
            {
                return false;
            }

            try
            {
                serialPort.WriteLine(cmd);
                return true;
            }
            catch (TimeoutException)
            {
                Debug.WriteLine("Write Timeout");
                serialPort.DiscardInBuffer();
            }
            catch (Exception e)
            {
                Debug.WriteLine("Write Exception : " + e.Message);
                serialPort.DiscardInBuffer();
            }
            return false;
        }

        private string ReadData(string cmd)
        {
            if (!serialPort.IsOpen)
            {
                return string.Empty;
            }

            try
            {
                // The return header of the command is "\r\n".
                serialPort.ReadTo(cmd + serialPort.NewLine);
                // THe return termination code is "\r\n\r\n".
                return serialPort.ReadTo(serialPort.NewLine + serialPort.NewLine);
            }
            catch (TimeoutException)
            {
                Debug.WriteLine("Read Timeout");
            }
            catch (Exception e)
            {
                Debug.WriteLine("Read Exception : " + e.Message);
            }
            finally
            {
                serialPort.DiscardInBuffer();
            }

            return string.Empty;
        }
    }
}
