﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2021. All rights reserved.
//
//*********************************************************

using MahApps.Metro.Controls;
using SampleDisplayControl.Communications;
using SampleDisplayControl.Watchers;
using System;
using System.Windows;
using System.Windows.Controls;

namespace SampleDisplayControl
{
    public partial class MainWindow : MetroWindow
    {
        private readonly MoverioWatcher moverioWatcher = new MoverioWatcher();
        private readonly DisplayControlCommunication communication = new DisplayControlCommunication();
        private string portName;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MetroWindow_Loaded(object sender, RoutedEventArgs e)
        {
            moverioWatcher.Add += MoverioWatcher_Add;
            moverioWatcher.Remove += MoverioWatcher_Remove;
            moverioWatcher.Start();
        }

        private void MetroWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            moverioWatcher.Add -= MoverioWatcher_Add;
            moverioWatcher.Remove -= MoverioWatcher_Remove;
            moverioWatcher.Stop();
        }

        private void MoverioWatcher_Add(object sender, MoverioInfo e)
        {
            if (string.IsNullOrEmpty(portName))
            {
                Dispatcher.BeginInvoke((Action)(() =>
                {
                    bool result = communication.Open(e.portName);
                    SetMoverioTypeDisplay(e.type);
                    Enable2D3DButtons(result);
                    EnableMuteButtons(result);
                    EnableAutoSleepButtons(result);
                    EnableManualSleepButtons(result);
                    EnablePixcellShiftControls(result);
                }
                ));
                portName = e.portName;
            }
        }

        private void MoverioWatcher_Remove(object sender, MoverioInfo e)
        {
            if (e.portName == portName)
            {
                communication.Close();
                portName = string.Empty;
                Dispatcher.BeginInvoke((Action)(() =>
                {
                    Enable2D3DButtons(false);
                    EnableMuteButtons(false);
                    EnableAutoSleepButtons(false);
                    EnableManualSleepButtons(false);
                    EnablePixcellShiftControls(false);
                }
                ));
            }
        }

        private void Get2D3D_Click(object sender, RoutedEventArgs e)
        {
            var mode = communication.GetMode();

            if (string.IsNullOrEmpty(mode))
            {
                Current2D3D.Text = "Error";
            }
            else
            {
                Current2D3D.Text = mode;
            }
        }

        private void Set2D3D_Click(object sender, RoutedEventArgs e)
        {
            Enable2D3DButtons(true);

            var button = sender as Button;
            var mode = button.Content as string;
            if (communication.SetMode(mode))
            {
                button.IsEnabled = false;
            }
        }

        private void GetMute_Click(object sender, RoutedEventArgs e)
        {
            var mode = communication.GetMute();

            if (string.IsNullOrEmpty(mode))
            {
                CurrentMute.Text = "Error";
            }
            else
            {
                CurrentMute.Text = mode;
            }
        }

        private void SetMute_Click(object sender, RoutedEventArgs e)
        {
            EnableMuteButtons(true);

            var button = sender as Button;
            var mode = button.Content as string;
            if (communication.SetMute(mode))
            {
                button.IsEnabled = false;
            }
        }

        private void GetAutoSleep_Click(object sender, RoutedEventArgs e)
        {
            var mode = communication.GetAutoSleep();

            if (string.IsNullOrEmpty(mode))
            {
                CurrentAutoSleep.Text = "Error";
            }
            else
            {
                CurrentAutoSleep.Text = mode;
            }
        }

        private void SetAutoSleep_Click(object sender, RoutedEventArgs e)
        {
            EnableAutoSleepButtons(true);

            var button = sender as Button;
            var mode = button.Content as string;
            if (communication.SetAutoSleep(mode))
            {
                button.IsEnabled = false;
            }
        }

        private void GetManualSleep_Click(object sender, RoutedEventArgs e)
        {
            var mode = communication.GetManualSleep();

            if (string.IsNullOrEmpty(mode))
            {
                CurrentManualSleep.Text = "Error";
            }
            else
            {
                CurrentManualSleep.Text = mode;
            }
        }

        private void SetManualSleep_Click(object sender, RoutedEventArgs e)
        {
            EnableManualSleepButtons(true);

            var button = sender as Button;
            var mode = button.Content as string;
            if (communication.SetManualSleep(mode))
            {
                button.IsEnabled = false;
            }
        }

        private void GetPixcellShift_Click(object sender, RoutedEventArgs e)
        {
            var mode = communication.GetPixcellShift();

            if (string.IsNullOrEmpty(mode))
            {
                CurrentPixcellShift.Text = "Error";
            }
            else
            {
                CurrentPixcellShift.Text = mode;
            }
        }

        private void SetPixcellShift_Click(object sender, RoutedEventArgs e)
        {
            var selectItem = ComboBoxPixcellShift.SelectedItem as ComboBoxItem;
            var selectValue = selectItem.Content.ToString();
            if (int.TryParse(selectValue, out int setValue))
            {
                communication.SetPixcellShift(setValue);
            }
        }

        private void SetMoverioTypeDisplay(MoverioType type)
        {
            var modeBT40 = (MoverioType.BT40 == type || MoverioType.BT45C == type )? Visibility.Visible : Visibility.Collapsed;
            StackPanelGetMute.Visibility = modeBT40;
            StackPanelSetMute.Visibility = modeBT40;
            StackPanelGetAutoSleep.Visibility = modeBT40;
            StackPanelSetAutoSleep.Visibility = modeBT40;
            StackPanelGetManualSleep.Visibility = modeBT40;
            StackPanelSetManualSleep.Visibility = modeBT40;
            StackPanelGetPixcellShift.Visibility = modeBT40;
            StackPanelSetPixcellShift.Visibility = modeBT40;

            if (type == MoverioType.BT40 || type == MoverioType.BT45C)
            {
                Height = 590;
            }
            else
            {
                Height = 150;
            }
        }

        private void Enable2D3DButtons(bool enable)
        {
            ButtonGet.IsEnabled = enable;
            Button2D.IsEnabled = enable;
            Button3D.IsEnabled = enable;
        }

        private void EnableMuteButtons(bool enable)
        {
            ButtonGetMute.IsEnabled = enable;
            ButtonMuteOn.IsEnabled = enable;
            ButtonMuteOff.IsEnabled = enable;
        }

        private void EnableAutoSleepButtons(bool enable)
        {
            ButtonGetAutoSleep.IsEnabled = enable;
            ButtonAutoSleepOn.IsEnabled = enable;
            ButtonAutoSleepOff.IsEnabled = enable;
        }

        private void EnableManualSleepButtons(bool enable)
        {
            ButtonGetManualSleep.IsEnabled = enable;
            ButtonManualSleepOn.IsEnabled = enable;
            ButtonManualSleepOff.IsEnabled = enable;
        }

        private void EnablePixcellShiftControls(bool enable)
        {
            ButtonGetPixcellShift.IsEnabled = enable;
            ComboBoxPixcellShift.IsEnabled = enable;
            ButtonPixcellShiftSet.IsEnabled = enable;
        }
    }
}
